#!/usr/bin/env python

#  Copyright (C) 2006-2011 Peter Robinson
#  Email: pjr@itee.uq.edu.au
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

""" This is a simple GUI that connects to the Pedro server and can be
used to manage subscriptions and send notifications.
The top window displays received notifications and P2P messages.
The second window displays a list of current subscriptions.
A Right-Click in this window will pop up a menu for managing subscriptions
and name registration.
A Left-Click will choose a subscrription.
"""

import sys
import wx
import wx.lib
import wx.lib.newevent
import thread
from pedroclient import *

port = 4550
machine = 'localhost'

help_text = "A right click in the second window will produce \
a menu with the following choices:\n\n\
* Add Subscription - add a subscription\n\
* Remove Subscription - remove highlighed subscription\n\
* Register - register a name with the Pedro server\n\
* Deregister - deregister name\n\
* Help - this message\n\n\
Notifications received are displayed in top window\n\n\
Notifications can be entered in the Notification: text entry\n\n\
Peer-to-peer messages can be sent using the bottom two text entries:\n\n\
* The P2P address has the form name@machine where name is a registered name \
and machine is the machine on which that process is running.\n\
* The Message is any Prolog term\n\n\
See the Pedro manual for examples of subscriptions and Prolog terms."

# Create an event for notifications
(NotificationEvent, EVT_NOTIFICATION_EVENT) = wx.lib.newevent.NewEvent()


def getNotifications(client, win):
    """Run as thread - post an event when notifications arrive"""
    
    while (True):
        msg,_ = client.get_notification()
        evt = NotificationEvent(notification=msg)
        wx.PostEvent(win, evt)
    

        
    
class MyApp(wx.App):

    """ The application."""
    
    def OnInit(self):
        frame = MyFrame("Pedro Client", (800, 600))
        frame.Show()
        self.SetTopWindow(frame)
        frame.setInitColSizes()
        return True

       
class MyFrame(wx.Frame):

    """ The main widget. """
    
    def __init__(self, title, s):
        wx.Frame.__init__(self, None, -1, title, size = s)
        self.panel = wx.Panel(self)
        # Text widget for displaying notifications
        self.tcl = wx.TextCtrl(self.panel, -1, "",
                               style = wx.TE_MULTILINE)
        wx.CallAfter(self.tcl.SetInsertionPoint, 0)
        
        # List for displaying and interacting with subscriptions
        self.list_ctrl = wx.ListCtrl(self.panel, -1,
                                     style=wx.SUNKEN_BORDER | wx.LC_REPORT)

        # Sending notifications
        notifybox = wx.BoxSizer(wx.HORIZONTAL)
        notifylabel = wx.StaticText(self.panel, -1, "Notification:  ")
        notifybox.Add(notifylabel, 0, wx.ALIGN_CENTRE|wx.ALL, 5)
        self.notifytext = wx.TextCtrl(self.panel, -1, "")
        notifybox.Add(self.notifytext, 1, wx.ALIGN_CENTRE|wx.ALL, 5)
        notifysend = wx.Button(self.panel, 10, "Send")
        notifybox.Add(notifysend, 0, wx.ALIGN_CENTRE|wx.ALL, 5)
        self.Bind(wx.EVT_BUTTON, self.ButtonClick, notifysend)
        
        # Sending P2P messages
        p2pbox = wx.BoxSizer(wx.HORIZONTAL)
        p2paddrlabel = wx.StaticText(self.panel, -1, "P2P Address:")
        p2pbox.Add(p2paddrlabel, 0, wx.ALIGN_CENTRE|wx.ALL, 5)
        self.p2paddrtext = wx.TextCtrl(self.panel, -1, "")
        p2pbox.Add(self.p2paddrtext, 1, wx.ALIGN_CENTRE|wx.ALL, 5)
        p2plabel = wx.StaticText(self.panel, -1, "Message:")
        p2pbox.Add(p2plabel, 0, wx.ALIGN_CENTRE|wx.ALL, 5)
        self.p2ptext = wx.TextCtrl(self.panel, -1, "")
        p2pbox.Add(self.p2ptext, 1, wx.ALIGN_CENTRE|wx.ALL, 5)
        
        p2psend = wx.Button(self.panel, 20, "Send")
        p2pbox.Add(p2psend, 0, wx.ALIGN_CENTRE|wx.ALL, 5)
        self.Bind(wx.EVT_BUTTON, self.ButtonClick, p2psend)

        # Lay the widgets out
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.tcl, 6, wx.EXPAND, 5)
        self.sizer.Add(self.list_ctrl, 3, wx.EXPAND, 5)
        self.sizer.Add(notifybox, 0, wx.EXPAND, 5)
        self.sizer.Add(p2pbox, 0, wx.EXPAND, 5)
        self.panel.SetSizer(self.sizer)
        self.panel.SetAutoLayout(True)


        # connect to Pedro server
        self.client = PedroClient(machine, port)
        self.subs = list()
        thread.start_new_thread(getNotifications, (self.client, self.panel))
        
        # Callback for notification event
        self.panel.Bind(EVT_NOTIFICATION_EVENT, self.OnNotification)
        
        # Set up subscription list display
        self.list_ctrl.InsertColumn(0, "ID")
        self.list_ctrl.InsertColumn(1, "Head")
        self.list_ctrl.InsertColumn(2, "Body")
        self.list_ctrl.SetColumnWidth(0, 30)
        
        self.currentItem = -1
        self.list_ctrl.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected,
                            self.list_ctrl)
        self.list_ctrl.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.OnItemDeselected,
                            self.list_ctrl)
        self.list_ctrl.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
        self.list_ctrl.Bind(wx.EVT_RIGHT_UP, self.OnRightUp)
        

    def ButtonClick(self, evt):
        """Button click handler."""
        
        bid = evt.GetId()
        if bid == 10:
            # send a notification
            notify = self.notifytext.GetValue().encode('latin-1')
            id = self.client.notify(notify)
            if (id == 0):
                dlg = wx.MessageDialog(self, "Invalid Notification", "Error",
                                       wx.OK | wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
            self.notifytext.Clear()
        elif bid == 20:
            # send a P2P message
            msg = self.p2ptext.GetValue().encode('latin-1')
            addr = self.p2paddrtext.GetValue().encode('latin-1')
            id = self.client.p2p(addr, msg)
            if (id == 0):
                dlg = wx.MessageDialog(self, "Invalid P2P Message", "Error",
                                       wx.OK | wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
            self.p2ptext.Clear()
        evt.Skip()

    def setInitColSizes(self):
        """Set up the initial column sizes for the list widget. """
        
        w = self.list_ctrl.GetSize()[0] - self.list_ctrl.GetColumnWidth(0)
        self.list_ctrl.SetColumnWidth(1, w/3 - 2)
        self.list_ctrl.SetColumnWidth(2, 2*w/3 - 2)
        
    def showSubs(self):
        """(Re)Display all the subscriptions in the list widget."""
        
        self.list_ctrl.DeleteAllItems()
        for sub in self.subs:
            self.list_ctrl.Append(sub)

    def OnItemSelected(self, evt):
        """ Select a subscription."""
        
        self.currentItem = evt.m_itemIndex

    def OnItemDeselected(self, evt):
        """ Deselect a subscription. """
        
        self.currentItem = -1

    def OnRightDown(self, evt):
        """ Select a subscription if click is over item. """
        
        x = evt.GetX()
        y = evt.GetY()
        self.currentItem = -1
        item, flags = self.list_ctrl.HitTest((x,y))
        if item != wx.NOT_FOUND and flags & wx.LIST_HITTEST_ONITEM:
            if self.currentItem != -1:
                self.list_ctrl.Select(self.currentItem, on=0)
            self.list_ctrl.Select(item)
        if not hasattr(self, "popupID1"):
            self.popupID1 = wx.NewId()
            self.popupID2 = wx.NewId()
            self.popupID3 = wx.NewId()
            self.popupID4 = wx.NewId()
            self.popupID5 = wx.NewId()
            
            self.Bind(wx.EVT_MENU, self.OnPopupOne, id = self.popupID1)
            self.Bind(wx.EVT_MENU, self.OnPopupTwo, id = self.popupID2)
            self.Bind(wx.EVT_MENU, self.OnPopupThree, id = self.popupID3)
            self.Bind(wx.EVT_MENU, self.OnPopupFour, id = self.popupID4)
            self.Bind(wx.EVT_MENU, self.OnPopupFive, id = self.popupID5)
            
        self.menu = wx.Menu()
        self.menu.Append(self.popupID1, "Add Subscription")
        self.menu.Append(self.popupID2, "Remove Subscription")
        self.menu.Append(self.popupID3, "Register")
        self.menu.Append(self.popupID4, "Deregister")
        self.menu.Append(self.popupID5, "Help")
        self.PopupMenu(self.menu)
        evt.Skip()
        
    def OnRightUp(self, evt):
        """ Destroy the menu."""
        self.menu.Destroy()




     
    def OnPopupOne(self, evt):
        """Make a subscription - pop up a dialog box to enter subscription."""
        
        dialog = wx.Dialog(self, -1, "Subscription",
                           style=wx.DEFAULT_DIALOG_STYLE)
        sizer = wx.BoxSizer(wx.VERTICAL)
        
        headbox = wx.BoxSizer(wx.HORIZONTAL)
        headlabel = wx.StaticText(dialog, -1, "Head: ")
        headbox.Add(headlabel, 0, wx.ALIGN_CENTRE|wx.ALL, 5)
        headtext = wx.TextCtrl(dialog, -1, "", size=(300,-1))
        headbox.Add(headtext, 1, wx.ALIGN_CENTRE|wx.ALL, 5)

        sizer.Add(headbox, 0, wx.GROW|wx.ALIGN_CENTRE_VERTICAL|wx.ALL, 5)
        
        goalbox = wx.BoxSizer(wx.HORIZONTAL)
        goallabel = wx.StaticText(dialog, -1, "Body: ")
        goalbox.Add(goallabel, 0, wx.ALIGN_CENTRE|wx.ALL, 5)
        goaltext = wx.TextCtrl(dialog, -1, "", size=(300,-1))
        goalbox.Add(goaltext, 1, wx.ALIGN_CENTRE|wx.ALL, 5)

        sizer.Add(goalbox, 0,
                  wx.GROW|wx.ALIGN_CENTRE_VERTICAL|wx.ALL, 5)

        btnsizer = wx.StdDialogButtonSizer()

        btn = wx.Button(dialog, wx.ID_OK)
        btn.SetDefault()
        btnsizer.AddButton(btn)
        
        btn = wx.Button(dialog, wx.ID_CANCEL)
        btnsizer.AddButton(btn)
        
        btnsizer.Realize()
        
        sizer.Add(btnsizer, 0,
                  wx.ALIGN_CENTRE_VERTICAL |wx.ALIGN_CENTRE_HORIZONTAL|wx.ALL,
                  5)

        dialog.SetSizer(sizer)
        sizer.Fit(dialog)
        headtext.SetFocus()
        dialog.CenterOnScreen()
        val = dialog.ShowModal()
        if val == wx.ID_OK:
            head = headtext.GetValue().encode('latin-1')
            body = goaltext.GetValue().encode('latin-1')
            id = self.client.subscribe(head, body)
            if (id == 0):
                dlg = wx.MessageDialog(self, "Invalid Subscription", "Error",
                                       wx.ID_OK | wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
            else:
                sub = list([id, head, body])
                self.subs.append(sub)
                self.showSubs()
        #dialog.Destroy()

    def OnPopupTwo(self, evt):
        """ Remove selected subscription."""
        
        if self.currentItem >= 0 and self.currentItem <= len(self.subs):
            subid = self.subs[self.currentItem][0]
            id = self.client.unsubscribe(subid)
            if id == 0:
                dlg = wx.MessageDialog(self, "Cannot Unsubscribe", "Error",
                                       wx.ID_OK | wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
            del self.subs[self.currentItem]
            self.showSubs()
            
    def OnPopupThree(self, evt):
        """Register name.

        - pop up dialog box, update window title

        """
        
        dialog = wx.Dialog(self, -1, "Registration",
                           style=wx.DEFAULT_DIALOG_STYLE)
        sizer = wx.BoxSizer(wx.VERTICAL)
        
        box = wx.BoxSizer(wx.HORIZONTAL)
        label = wx.StaticText(dialog, -1, "Name: ")
        box.Add(label, 0, wx.ALIGN_CENTRE|wx.ALL, 5)
        text = wx.TextCtrl(dialog, -1, "", size=(300,-1))
        box.Add(text, 1, wx.ALIGN_CENTRE|wx.ALL, 5)
        sizer.Add(box, 0, wx.GROW|wx.ALIGN_CENTRE_VERTICAL|wx.ALL, 5)

        btnsizer = wx.StdDialogButtonSizer()

        btn = wx.Button(dialog, wx.ID_OK)
        btn.SetDefault()
        btnsizer.AddButton(btn)
        
        btn = wx.Button(dialog, wx.ID_CANCEL)
        btnsizer.AddButton(btn)
        
        btnsizer.Realize()
        
        sizer.Add(btnsizer, 0,
                  wx.ALIGN_CENTRE_VERTICAL |wx.ALIGN_CENTRE_HORIZONTAL|wx.ALL,
                  5)

        dialog.SetSizer(sizer)
        sizer.Fit(dialog)
        text.SetFocus()
        dialog.CenterOnScreen()
        val = dialog.ShowModal()
        if val == wx.ID_OK:
            name = text.GetValue().encode('latin-1')
            id = self.client.register(name)
            if (id == 0):
                dlg = wx.MessageDialog(self, "Invalid Registration", "Error",
                                       wx.ID_OK | wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
            else:
                self.SetTitle("PedroClient: " + name)

    def OnPopupFour(self, evt):
        """ Deregister name - reset window name."""
        
        id = self.client.deregister()
        if id == 0:
            dlg = wx.MessageDialog(self, "Cannot Deregister", "Error",
                                   wx.ID_OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
        self.SetTitle("PedroClient")

    def OnPopupFive(self, evt):
        """ Display help. """
        
        frame = wx.Frame(self, -1, "Help", size = (500,400))
        text = wx.StaticText(frame, 1, help_text)
        frame.Show()
 
    def OnNotification(self, evt):
        """Display notification."""
        
        self.tcl.write(evt.notification)
        self.tcl.write("\n")



        
if  __name__ == '__main__':
    argc = len(sys.argv)
    if argc > 1:
        i = 1
        while i < argc:
            if sys.argv[i] == '-P':
                i = i+1
                port = int(sys.argv[i])
            elif sys.argv[i] == '-M':
                i = i+1
                machine = sys.argv[i]
            else:
                print "Usage: pedro_gui.py [-P port] [-M machine]"
                print ""
                print help_text
                sys.exit()
            i = i+1
    app = MyApp(False)
    app.MainLoop()

